"""
Implementation for the German operator metronom Eisenbahngesellschaft mbH.

Contains the following APIs:

- ``MetronomCaptivePortal``

---
"""
from __future__ import annotations

from .apis import MetronomCaptivePortal

__all__ = [
    "MetronomCaptivePortal",
]
