"""
Implementation of the Italian operator Trenitalia S.p.A.

Contains the following APIs:

- ``PortaleRegionale``

---
"""
from __future__ import annotations

from .apis import PortaleRegionale

__all__ = [
    "PortaleRegionale",
]
