"""
Implementation of the German operator Flix Train GmbH.

Contains the following APIs:

- ``FlixTainment``

---
"""
from __future__ import annotations

from .apis import FlixTainment

__all__ = [
    "FlixTainment"
]
