"""
Package for third parties that provide API services and are used by other operators.

Contains the following third parties:

- ``onboardapis.train.third_party.icomera`` - Icomera AB
- ``onboardapis.train.third_party.unwired`` - Unwired Networks GmbH

---
"""
from __future__ import annotations

__all__ = [
    "icomera",
    "unwired",
]
