"""
Implementation for the German operator Transdev Hannover GmbH.

Contains the following APIs:

- ``SBahnHannover``

---
"""
from __future__ import annotations

from .apis import SBahnHannover

__all__ = [
    "SBahnHannover",
]
