"""
Package for German trains.

Contains the following operators:

- ``onboardapis.train.de.bth`` - ALSTOM Transportation Germany GmbH
- ``onboardapis.train.de.db`` - Deutsche Bahn / DB AG
- ``onboardapis.train.de.flix`` - Flix Train GmbH
- ``onboardapis.train.de.me`` - metronom Eisenbahngesellschaft mbH
- ``onboardapis.train.de.tdh`` - Transdev Hannover GmbH

---
"""
from __future__ import annotations

__all__ = [
    "bth",
    "db",
    "flix",
    "me",
    "tdh",
]
