"""
Implementation of the Austrian operator Österreichische Bundesbahnen.

Contains the following APIs:

- ``RailnetRegio``

---
"""
from __future__ import annotations

from .apis import RailnetRegio

__all__ = [
    "RailnetRegio",
]
