"""
Implementation for the French train operator SNCF Voyageurs.

Contains the following APIs:

- ``PortalINOUI``

---
"""
from __future__ import annotations

from .apis import PortalINOUI

__all__ = [
    "PortalINOUI",
]
