"""
Package for Austrian trains.

Contains the following operators:

- ``onboardapis.train.at.obb`` - Österreichische Bundesbahnen

---
"""
from __future__ import annotations

__all__ = [
    "obb",
]
